%  seeRidge2D.m:
%  matlab script to generate solutions and plots for the 2-D spectral Ridge flow code
%

% let's scale distance to the Extraction depth L=sqrt(\eta U0/\delta rho g)
close all;
PRINT=true;

L = 10; % use 10 km (about 3x10^20 Pa s at 5 cm/yr)
zMax = 60/L;  % set depth to 60 Km

% set bounds for periodic domain
xPeriodicMin = -4*zMax;
xPeriodicMax = 4*zMax;
xPeriodicBounds = [ xPeriodicMin xPeriodicMax ];

% set bounds for solution domain
xRidgeMin = -zMax;
xRidgeMax = zMax;
xRidgeBounds = [ xRidgeMin  xRidgeMax ];
depthRidge = linspace(0,zMax);

% get upper velocity boundary condition

nPoints = 1024; % number of points in horizontal FFT
lambda.ridge = .1;  % taper half-width at the ridge axis
lambda.taper = .4;  % taper half-width at the periodic boundaries
uTop = setTopVelocity2D(xPeriodicBounds,nPoints,xRidgeBounds,lambda);

% plot the upper boundary condition

figure;
plot(uTop.xPeriodic,uTop.uXPeriodic);
grid
set(gca,'fontsize',14,'fontweight','bold','YLim',[-1.1 1.1]);
xlabel('x Distance');
ylabel('Surface horizontal velocity');
title(sprintf('Surface velocity Boundary condition 2D Ridge model: n=%d',nPoints));

% calculate the spectral solution over the clipped domain

tic;
ridge2D = specRidge(uTop,depthRidge);
toc

% and plot

figure;
subplot(2,2,1); imagesc(ridge2D.x,ridge2D.z,ridge2D.u); colorbar('SouthOutside'); axis image;
  title(sprintf('U: [max min] = %g, %g',max(max(ridge2D.u)),min(min(ridge2D.u))));
subplot(2,2,2); imagesc(ridge2D.x,ridge2D.z,ridge2D.w); colorbar('SouthOutside'); axis image;
  title(sprintf('W: [max min] = %g, %g',max(max(ridge2D.w)),min(min(ridge2D.w))));
subplot(2,2,3); imagesc(ridge2D.x,ridge2D.z,ridge2D.px); caxis([-1 1]); colorbar('SouthOutside'); axis image;
  title(sprintf('P_x: [max min] = %g, %g',max(max(ridge2D.px)),min(min(ridge2D.px))));
subplot(2,2,4); imagesc(ridge2D.x,ridge2D.z,ridge2D.pz);  caxis([-1 1]); colorbar('SouthOutside'); axis image;
  title(sprintf('P_z: [max min] = %g, %g',max(max(ridge2D.pz)),min(min(ridge2D.pz))));
  
figure; % pressure
imagesc(ridge2D.x,ridge2D.z,ridge2D.p); 
caxis([-5 0]);
colorbar('SouthOutside'); axis image;
set(gca,'fontsize',14,'fontweight','bold','Xlim',[-1 1],'YLim',[0 1]);
xlabel('x Distance');
ylabel('Depth');
title(sprintf('Dynamic Pressure: [max min] = %g, %g',max(max(ridge2D.p)),min(min(ridge2D.p))));


if (PRINT)
   figure(1);
   eval(sprintf('print -depsc2 specRidge2D_UTop_N_%d_Lambda_%g.eps',nPoints,lambdaRidgeAxis));
   figure(2);
   eval(sprintf('print -depsc2 specRidge2D_U_GradP_N_%d_Lambda_%g.eps',nPoints,lambdaRidgeAxis));
   figure(3);
   eval(sprintf('print -depsc2 specRidge2D_P_N_%d_Lambda_%g.eps',nPoints,lambdaRidgeAxis));
end
