%  seeRidge3D.m:
%  matlab script to generate solutions and plots for the 3-D spectral Ridge flow code
%

% let's scale distance to the Extraction depth L=sqrt(\eta U0/\delta rho g)
close all;
PRINT=true;

L = 10; % use 10 km (about 3x10^20 Pa s at 5 cm/yr)
zMax = 60/L;  % set depth to 60 Km

% set bounds for local domain
xMin = -zMax; xMax = zMax;
yMin = 0    ; yMax = 2*zMax;
xyBounds = [ xMin xMax ; yMin yMax];

% set number of grid-points
nx = 128; 
ny = 128;
nPoints = [ nx ny];

% set Ridge-Transform-Ridge Geometry
ridgeCoords.xSegment = zMax*[ -.25 .25];
ridgeCoords.yTransform = zMax*1;

% set size of global xPeriodic domain

xPeriodicScale = 4;

% set appropriate tapers
lambda.ridge = 0.25;
lambda.transform = .25;
lambda.taper = .5;

% set depth of solution domain
depthRidge = linspace(0,zMax,64);

% get upper velocity boundary condition
uTop = setTopVelocity3D(xyBounds,nPoints,ridgeCoords,xPeriodicScale,lambda);

% plot the upper boundary condition in 2-D

figure;
imagesc(uTop.x,uTop.yPeriodic,uTop.u); axis image; axis xy;
set(gca,'fontweight','bold','fontsize',14);
xlabel('x');
ylabel('y');
title(sprintf('Surface velocity Boundary condition 3D Ridge model: n=%d,%d',size(uTop.u)'));

% plot the upper boundary condition for large xPeriodic Domain

figure
plot(uTop.xPeriodic,uTop.uXPeriodic);
grid
set(gca,'fontsize',14,'fontweight','bold','YLim',[-1.1 1.1]);
xlabel('x Distance');
ylabel('Surface horizontal velocity');
title(sprintf('Surface velocity Boundary condition 3D Ridge model: n=%d',length(uTop.xPeriodic)));


% calculate the spectral solution over the clipped domain

tic;
ridge3D = specRidge(uTop,depthRidge);
toc

% and plot

[X,Y,Z] = meshgrid(ridge3D.x,ridge3D.y,ridge3D.z);

% slices
xSlice = zMax*[ 0 .5];
ySlice = zMax*[0 1 1.5];
zSlice = zMax*[ 0 1];

% U velocity
figure;
slice(X,Y,Z,ridge3D.u,xSlice,ySlice,zSlice); shading interp; axis image;
set(gca,'zDir','reverse');
cameratoolbar;
title('U');

% V velocity
figure;
slice(X,Y,Z,ridge3D.v,xSlice,ySlice,zSlice); shading interp; axis image
set(gca,'zDir','reverse');
cameratoolbar;
title('V');

% W velocity
figure;
slice(X,Y,Z,ridge3D.w,xSlice,ySlice,zSlice); shading interp; axis image;
set(gca,'zDir','reverse');
cameratoolbar;
title('W');

% P Pressure
figure;
slice(X,Y,Z,ridge3D.p,[],zMax*[.9 1.1],zSlice); shading interp; axis image
set(gca,'zDir','reverse','fontweight','bold','fontsize',14);
%cameratoolbar;
title('P');

hold on;
%
%  add some streamlines
xs = linspace(min(ridge3D.x),max(ridge3D.x),10);
ys = ones(size(xs));
zs = max(ridge3D.z)*ys;
yy = linspace(min(ridge3D.y),max(ridge3D.y),5);
startx=[];
starty=[];
startz=[];
for i=1:length(yy)
   startx=[ startx xs];
   starty=[ starty ys*yy(i) ];
   startz=[ startz zs];
end

streamline(X,Y,Z,ridge3D.u,ridge3D.v,ridge3D.w,startx,starty,startz)
az=-25.5; el=16;
view(az,el);

if (PRINT)
   nz = length(depthRidge);
   figure(1);
   eval(sprintf('print -depsc2 specRidge3D_UTop_N_%dx%dx%d.eps',nPoints,nz));
%  figure(3);
%  eval(sprintf('print -depsc2 specRidge3D_U_N_%dx%dx%d.eps',nPoints,nz));
   figure(6);
   eval(sprintf('print -depsc2 specRidge3D_P_N_%dx%dx%d_01.eps',nPoints,nz));
   az=-21.5; el=-11;
   view(az,el)
   eval(sprintf('print -depsc2 specRidge3D_P_N_%dx%dx%d_02.eps',nPoints,nz));
end
