function uTop = setTopVelocity2D(xPeriodicBounds,nPoints,xRidgeBounds,lambda)
% SETTOPVELOCITY2D - Sets velocity for 2D ridge problem to tapered error function
%   function uTop = setTopVelocity2D(xPeriodicBounds,nPoints,xDomainBounds,lambda)
%
%   sets velocity to topVelocity = erf(x/lambda.ridge) for central part with 
%        tanh tapers at beginning and end with half-width lambda.taper
%
%   xPeriodicBounds:  min,max coordinates for large periodic domain.  Assumes f(xMin)=f(xMax);
%   nPoints:  Number of panels size dx in periodic domain nPoints*dx = xMax-xMin;
%   xRidgeBounds: [ xMin xMax ] for sub-domain over which the solution is returned
%   lambda.ridge:  taper half-width for ridge axis u~ erf(x/lambdaRidge) near x=0;
%   lambda.taper:  taper half-width at ends of periodic domain for smooth fourier spectrum
%
%   uTop:  Structure containing informatin about horizontal velocity at surface of the domain
%          uTop.nDim=2  Boundary condition for 2D problem
%          uTop.uXPeriodic 1-D array of full velocity over entire periodic domain
%          uTop.xPeriodic  array of coordinates for full periodic domain
%          uTop.x          array of coordinates for sub-domain
%          uTop.indexPeriodic index of coordinates such that uTop.x=uTop.xPeriodic(uTop.indexPeriodic)
   
   
   % 2D Boundary conditions
   
   uTop.nDim =2;
   
   % Full domain coordinates
   
   xMin = xPeriodicBounds(1);
   xMax = xPeriodicBounds(2);
   x = linspace(xMin,xMax,nPoints+1);  % full period
   x = x(1:end-1);  % remove redundant last point for input to ffts
   uTop.xPeriodic = x; % set full domain
   
   % set velocity BC
   
   uTop.uXPeriodic = erf(x/lambda.ridge);
   
   %  add end tapers
   
   uTop.uXPeriodic = max(uTop.uXPeriodic,-tanh((x-xMin)/lambda.taper)); % left taper
   uTop.uXPeriodic = min(uTop.uXPeriodic,-tanh((x-xMax)/lambda.taper)); % right taper
   
   % calculate sub-domain index and coordinates
   
   uTop.indexPeriodic = find(uTop.xPeriodic>=xRidgeBounds(1) & uTop.xPeriodic <= xRidgeBounds(2));
   uTop.x = uTop.xPeriodic(uTop.indexPeriodic);
   
   
   
   
