function uTop = setTopVelocity3D(xyBounds,nPoints,ridgeCoords,xPeriodicScale,lambda)
% SETTOPVELOCITY3D - Sets velocity for 3D ridge-transform-ridge problem  
%   function uTop = setTopVelocity2D(xyBounds,nPoints,ridgeCoords,xPeriodicScale,lambda)
%
%   sets upper  Velocity BC to tapered single ridge-transform-ridge condition set in a larger xPeriodic domain and
%        periodic in y
%
%   xyBounds:  2-D array [ xMin xMax ; yMin yMax]  coordinates for clipped ridge domain
%   nPoints:  [ nx ny ] vector of number of panels size dx,dy in top domain nPoints*dx = xMax-xMin;
%   ridgeCoords:    structure containing xCoordinates of 2 ridge segments, and y coordinate of the transform
%                  ridgeCoords.xSegment  ridgeCoords.yTransform
%   xPeriodicScale: scale factor for periodic x boundaries  (i.e. xPeriodicScale*(xMax - xMin);)
%
%   lambda: structure with taper half-widths for ridges, transforms and periodic xboundaries
%
%   uTop:  Structure containing informatin about horizontal velocity at surface of the domain
%                  uTop.nDim = 3; Boundary condition for 3D problem
%                  uTop.u   2-D array with u(x,y,0) for top boundary (assumed periodic in y)
%                  uTop.uXPeriodic 1-D array of mean velocity over entire periodic x domain
%                  uTop.indexPeriodic integer indices such that uTop.uXPeriodic(index) = mean(uTop.u) 
%                  uTop.x  x coordinates for clipped 3-D domain
%                  uTop.y  y coordinates for clipped 3-D domain
%                  utop.xPeriodic coordinates for extended 2-D periodic domain in x
%                  utop.yPeriodic y coordinates for extended 2-D periodic domain in y

   % set flag for 3D Boundary conditions
   
   uTop.nDim = 3;
   
   % clipped domain coordinates
   xMin = xyBounds(1,1); xMax = xyBounds(1,2);  
   yMin = xyBounds(2,1); yMax = xyBounds(2,2);
   x0=mean(xyBounds(1,:)); % mid-point
   ni = nPoints(1); nj = nPoints(2);
   
   x = linspace(xMin,xMax,ni+1); % x coordinates  
   y = linspace(yMin,yMax,nj+1); % y coordinates

   xMinPeriodic = x0 - xPeriodicScale*(xMax-xMin)/2;
   xMaxPeriodic = x0 + xPeriodicScale*(xMax-xMin)/2;
   xPeriodic = linspace(xMinPeriodic,xMaxPeriodic,nPoints(1)*xPeriodicScale+1);
   
   [xIntersect,iX,iXPeriodic]=intersect(x,xPeriodic);  % find the overlap region in xPeriodic (dangerous this way)
      
   if ( isempty(iXPeriodic) ) % check for stupidity and die
      error('cant find exact overlap region');
   end
   
     
   
   %
   % fill in top velocity U(x,y,0) with smoothed Ridge-Transform-Ridge solution
   %
   
   uTop.u = zeros(nj+1,ni+1); % allocate size
   x1 = ridgeCoords.xSegment(1); % locations of ridge segments and transform
   x2 = ridgeCoords.xSegment(2);
   yT = ridgeCoords.yTransform;
   
   uYMin = erf((x-x1)/lambda.ridge);  % set velocities on y=yMin and yMax
   uYMax = erf((x-x2)/lambda.ridge);
   
   % Bit of black magic from setutop3d03.f to put in y smoothed variability that matches boundaries
   
   [X,Y] = meshgrid(x,y);
   f = @(y) erf((y-yT)./lambda.transform);
   
   yScale = (f(Y) - f(yMin))./(f(yMax)-f(yMin));
   e=ones(length(y),1);
   uTop.u = e*uYMin + (e*(uYMax-uYMin)).*yScale;
   
   % and reflect in y to make periodic
   
   uTop.u = [ uTop.u ; uTop.u(end-1:-1:2,:) ]; % reflect
   dy = y(2)-y(1);
   uTop.yPeriodic = yMin+dy*[0:2*nj-1];
   
   % Now set large scale x-Periodic velocity
   
   uMean = mean(uTop.u);
   uTop.uXPeriodic = zeros(size(xPeriodic));
   uTop.uXPeriodic(iXPeriodic) = uMean;
   uTop.uXPeriodic(1:min(iXPeriodic)-1) = uMean(1);
   uTop.uXPeriodic(max(iXPeriodic)+1:end) = uMean(end);
   uTop.uXPeriodic = max(uTop.uXPeriodic,-tanh((xPeriodic-xMinPeriodic)/lambda.taper)); % left taper
   uTop.uXPeriodic = min(uTop.uXPeriodic,-tanh((xPeriodic-xMaxPeriodic)/lambda.taper)); % right taper
   
   % and trim redundant points for inputs to FFTs
   
   uTop.u = uTop.u(:,1:end-1);
   uTop.uXPeriodic = uTop.uXPeriodic(1:end-1);
   uTop.indexPeriodic = iXPeriodic(1:end-1);
   uTop.x = x(1:end-1);
   uTop.y = y;
   uTop.xPeriodic = xPeriodic(1:end-1);
   
   

   
