function ridgeStruct2D = specRidge2D(uTop,depthRidge)
% SPECRIDGE2D - returns velocity, pressure, P, \Grad P and coordinates for 2-D spectral ridge code
%   
%     ridgeStruct2D = specRidge2D(uTop,depthRidge)
%
%     Calculates: solid flow field and pressure for incompressible
%                 Stoke's flow in 2 Dimensions for an
%                 iso-viscous half-space driven by imposed flow on the upper boundary.
%                 This calculation uses a Discrete Fourier Transform of the upper boundary condition 
%                       to solve on a large periodic domain  given by uTop.xPeriodic
%                       then solves for velocities and pressures on a smaller domain in uTop.x and  depthRidge in z
%                 Uses ffts in the horizontal so care needs to be taken to enforce periodicity
%                 and number of grid points (see setTopVelocity.m)
%                 
%
%     uTop:     structure set in setTopVelocity2D.m 
%          uTop.uXPeriodic 1-D array of full velocity over entire periodic domain
%          uTop.xPeriodic  array of coordinates for full periodic domain
%          uTop.x          array of coordinates for sub-domain
%          uTop.indexPeriodic index of coordinates such that uTop.x=uTop.xPeriodic(uTop.indexPeriodic)
%     depthRidge:      vector of depth coordinates (0 is the surface, z increases downwards)
%
%     ridgeStruct2D:  output structure for a 2-D ridge calculation
%                     ridgeStruct2D.x:  x coordinates of solution domain
%                     ridgeStruct2D.z:  z coordinates of solution domain
%                     ridgeStruct2D.u:  horizontal velocity
%                     ridgeStruct2D.w:  vertical velocity
%                     ridgeStruct2D.p:  scalar dynamic pressure field
%                     ridgeStruct2D.px: horizontal dynamic pressure gradient (Delsq U)
%                     ridgeStruct2D.pz: vertical   dynamic pressure gradient (Delsq W)


   n = length(uTop.uXPeriodic);  % number of points in dft 
   if (mod(n,2)) %check that n is even
      warning('Use even number of points for dft');
   end
   
   % calculate domain sizes 
   
   dx = uTop.x(2)-uTop.x(1); % uniform grid spacing
      
   ridgeStruct2D.x = uTop.x;
   ridgeStruct2D.z = depthRidge;
   mRidge = length(ridgeStruct2D.z);
   nRidge = length(ridgeStruct2D.x);
   
   % allocate velocity and pressure arrays
   ridgeStruct2D.u = zeros(mRidge,nRidge);
   ridgeStruct2D.w = zeros(mRidge,nRidge);
   ridgeStruct2D.p = zeros(mRidge,nRidge);
   ridgeStruct2D.px = zeros(mRidge,nRidge);
   ridgeStruct2D.pz = zeros(mRidge,nRidge);
   
   % calculate wave vector
   
   kNyquist=2.*pi/dx;  %  Nyquist frequency
   k = (kNyquist/n)*[0:n/2 -(n/2)+1:-1];  % calculate positive and negative wave numbers
   kAbs = abs(k);  % also need just absolute k for z dependence
   
   % calculate fourier transform of top velocity
   
   uHat = fft(uTop.uXPeriodic);
   
   % now loop over z coordinates,  calculate velocity and Pressure and clip to sub-domain
   
   iClip = uTop.indexPeriodic;
   
   for j=1:mRidge
      z=depthRidge(j);
      ekz = exp(-kAbs*z);
      Uz = ifft(uHat.*(1-kAbs*z).*ekz);
      Wz = ifft(-i*k.*uHat.*z.*ekz);
      Pz = ifft(-2*i*k.*uHat.*ekz);
      delSquaredUz = ifft(  2.*uHat.*kAbs.*kAbs.*ekz);
      delSquaredWz = ifft(i*2.*uHat.*k.*kAbs.*ekz);
      ridgeStruct2D.u(j,:) = real(Uz(iClip));
      ridgeStruct2D.w(j,:) = real(Wz(iClip));
      ridgeStruct2D.p(j,:) = real(Pz(iClip));
      ridgeStruct2D.px(j,:) = real(delSquaredUz(iClip));
      ridgeStruct2D.pz(j,:) = real(delSquaredWz(iClip));
   end
      
