function ridgeStruct3D = specRidge3D(uTop,depthRidge)
% SPECRIDGE2D - returns velocity, pressure, \Grad P and coordinates for 3-D spectral ridge code
%   
%     ridgeStruct3D = specRidge3D(uTop,depthRidge)
%
%     Calculates: solid flow field and pressure for incompressible
%                 Stoke's flow in 3 Dimensions for an
%                 iso-viscous half-space driven by imposed flow on the upper boundary.
%                 This calculation first uses specRidgeD to solve for the mean 2-D flow  
%                        on a larger periodic domain  given by xBoundsPeriodic
%                       then solves in 3-D  for the perturbations in the   for velocities and pressures on a smaller domain from
%                        [xBoundsRidge(1) xBoundsRidge(2)] in x and coordinates depthRidge in z
%                 Uses ffts in the horizontal so care needs to be taken to enforce periodicity
%                 and number of grid points (see setTopVelocity.m)
%                 
%
%     uTop:     Structure produced from setTopVelocity3D 
%                  uTop.nDims =3; check for 3D Bcs
%                  uTop.u   2-D array with u(x,y,0) for top boundary
%                  uTop.uXPeriodic 1-D array that contains mean u(x)@z=0 
%                  uTop.indexPeriodic integer indices such that uTop.uXPeriodic(index) = mean(uTop.u) 
%                  uTop.x  x coordinates for 3-D domain
%                  uTop.y  y coordinates for 3-D domain
%                  utop.xPeriodic coordinates for extended 2-D periodic domaincontaining  U(x,y,0) of horizontal surface velocity. Full solution is assumed to reflect across yMax face
%                  utop.yPeriodic
%     depthRidge:      vector of depth coordinates (0 is the surface, z increases downwards)
%
%     ridgeStruct3D:  output structure for a 3-D ridge calculation
%                     ridgeStruct3D.x:  x coordinates of solution domain
%                     ridgeStruct3D.y:  y coordinates of solution domain
%                     ridgeStruct3D.z:  z coordinates of solution domain
%                     ridgeStruct3D.u:  x velocity
%                     ridgeStruct3D.v:  y velocity
%                     ridgeStruct3D.w:  vertical velocity
%                     ridgeStruct3D.p:  scalar dynamic pressure field
%                     ridgeStruct3D.px: x dynamic pressure gradient (Delsq U)
%                     ridgeStruct3D.py: y dynamic pressure gradient (Delsq V)
%                     ridgeStruct3D.pz: vertical   dynamic pressure gradient (Delsq W)
   
% check for appropriate uTop structure
   if (uTop.nDim ~= 3 )
      error('BC structure must be 3-D');
   end
   
   % set coordinates and Allocate space for output 
   
   [X,Y,Z]=meshgrid(uTop.x,uTop.y,depthRidge);
   v0=zeros(size(X));  % note dimensions of X,v0 etc are [ ny, nx, nz ]
   
   ridgeStruct3D.x = uTop.x;
   ridgeStruct3D.y = uTop.y;
   ridgeStruct3D.z = depthRidge;
   
   ridgeStruct3D.u = v0;
   ridgeStruct3D.v = v0;
   ridgeStruct3D.w = v0;
   ridgeStruct3D.p = v0;
   ridgeStruct3D.px = v0;
   ridgeStruct3D.py = v0;
   ridgeStruct3D.pz = v0;
   
   
% first calculate 2-D (x,z) solution and fill in for u,w,p,px,pz

   ridge2D = specRidge2D(uTop,depthRidge);
   for j=1:length(uTop.y)
      ridgeStruct3D.u(j,:,:)=ridge2D.u';
      ridgeStruct3D.w(j,:,:)=ridge2D.w';
      ridgeStruct3D.p(j,:,:)=ridge2D.p';
      ridgeStruct3D.px(j,:,:)=ridge2D.px';
      ridgeStruct3D.pz(j,:,:)=ridge2D.pz';
   end
   
   % set up top boundary condition for 2D fft by subtracting 2-D solution
   
   U = uTop.u;
   [m,n]=size(U);
   U = U-ones(m,1)*uTop.uXPeriodic(uTop.indexPeriodic);  % subtract 2-D solution
   
   % calculate grid spacing parameters 
   
   dx = uTop.x(2)-uTop.x(1); % uniform grid spacing
   dy = uTop.y(2)-uTop.y(1); 
   
   % calculate wave vectors for fft
   
   kNyquist=2.*pi./[dx dy];  %  Nyquist frequencies
   kx = (kNyquist(1)/n)*[0:n/2 -(n/2)+1:-1];  
   ky = (kNyquist(2)/m)*[0:m/2 -(m/2)+1:-1];  
   [ KX KY ] = meshgrid(kx,ky); % wave vector matrices
   k = sqrt(KX.*KX+KY.*KY);  % scalar wavenumber
   k(1,1) = eps*(1+i); % be careful about DC component
   
   % calculate fourier transform of top velocity
   
   uHat = fft2(U);
   uHat(1,1)=0+0*i; % 0 any residual DC component
   
   % now loop over z coordinates and calculate velocity and Pressure, clip and add
   
   yClip = 1:length(uTop.y); % index of y points for clipped domain
   
   for j=1:length(depthRidge)
      z=depthRidge(j);
      ekz = exp(-k*z);
      kxz = KX*z;
      Uz = ifft2(uHat.*(1-KX.*kxz./k).*ekz);
      Vz = ifft2(-uHat.*KY.*kxz./k.*ekz);    % check sign on this
      Wz = ifft2(-i*uHat.*kxz.*ekz);
      Pz = ifft2(-2*i*KX.*uHat.*ekz);
      delSquaredUz = ifft2(  2.*uHat.*KX.*KX.*ekz);
      delSquaredVz = ifft2(  2.*uHat.*KX.*KY.*ekz);
      delSquaredWz = ifft2(i*2.*uHat.*k.*KX.*ekz);
      ridgeStruct3D.u(:,:,j) = ridgeStruct3D.u(:,:,j)+real(Uz(yClip,:));
      ridgeStruct3D.v(:,:,j) = ridgeStruct3D.v(:,:,j)+real(Vz(yClip,:));
      ridgeStruct3D.w(:,:,j) = ridgeStruct3D.w(:,:,j)+real(Wz(yClip,:));
      ridgeStruct3D.p(:,:,j) = ridgeStruct3D.p(:,:,j)+real(Pz(yClip,:));
      ridgeStruct3D.px(:,:,j) = ridgeStruct3D.px(:,:,j)+real(delSquaredUz(yClip,:));
      ridgeStruct3D.py(:,:,j) = ridgeStruct3D.py(:,:,j)+real(delSquaredVz(yClip,:));
      ridgeStruct3D.px(:,:,j) = ridgeStruct3D.pz(:,:,j)+real(delSquaredWz(yClip,:));
   end
      
   
