c*******************************************************************
c     routine to calculate 3-d velocity and pressure field (delsqV)
c     given  Fourier transform solution for 3-D flow in a half-space
c     (i.e. essentially Phipps-Morgan and forsyth)
c
c     Inputs surface velocity field Utop (assumed full periodic, nit,
c     njt)
c
c     outputs 3-D velocity field and Delsqu V on 3-D lattice (ni,nj,nk)
c     Uses the phipps-morgan trick of subracting and adding a large wave 
c     length 2-D solution
c
c     for multi-grid applications it will usually be assumed that
c      ni,nj,nk are 2^n+1 style grids while nit,njt and ni2t will be
c      power of 2 grids
c*******************************************************************

      subroutine flow3dpass01(v,d2v,ni,nj,nk,utop,nit,njt,ni2t,wk,dx,dy
     &     ,dz)
      implicit none
      integer ni,nj,nk,nit,njt,ni2t
      real v(3,ni,nj,nk),d2v(3,ni,nj,nk)
      real utop(ni,nj)
      real wk(2*ni2t+15*nit*njt)
      real dx,dy,dz

      real x
      integer i
      integer wp(6),ip,p1,p2,p3,p4,p5,p6 ! pointers
      integer i0,i0t,i02
      real zero
      data zero /0./

c----
c     set up pointers for work array
c----
      wp(1)=1                   ! top of 2-D solution (complex)
      wp(2)=wp(1)+2*ni2t          ! utop-2-D solution complex
      wp(3)=wp(2)+ 2*nit*njt    ! complex gunk for inverting V's
      wp(4)=wp(3)+6*nit*njt     ! complex gunk for inverting d2V's
      wp(5)=wp(4)+6*nit*njt     ! wkspace for k's
      wp(6)=wp(5)+nit     ! wkspace for k's
      p1=wp(1)
      p2=wp(2)
      p3=wp(3)
      p4=wp(4)
      p5=wp(5)
      p6=wp(6)

c---
c   zero  arrays
c---
      call arrcon(wk,zero,p4)
      call arrcon(v,zero,3*ni*nj*nk)
      call arrcon(d2v,zero,3*ni*nj*nk)
c---
c     set up 2-D solution and subtract from utop
c---
      i0=ni/2+1
      i02=ni2t/2+1
      i0t=nit/2+1
      do i=1,ni2t
         x=dx*(i-i02)
         ip=wp(1)+2*(i-1)
         wk(ip)=tanh(.25*x/dx)
      enddo
      do i=1,20                 !taper beginning and end
         x=dx*(i-1)
         ip=wp(1)+2*(i-1)
         wk(ip)=max(wk(ip),-tanh(.5*x/dx))
      enddo
      do i=ni2t-19,ni2t
         x=dx*(i-ni2t-1)
         ip=wp(1)+2*(i-1)
         wk(ip)=min(wk(ip),-tanh(.5*x/dx))
      enddo
      call setutop(wk(p2),utop,wk(p1),nit,njt,ni2t,i0t,i02)
      call arrvget(wk(p2),utop,1,2,nit*njt)
c---
c     calculated 2-D solution and copy onto v and d2v
c---      
      call flow2d(v,d2v,wk(p1),wk(p3),wk(p4),wk(p5),ni,nj,nk,ni2t,dx,dz
     &     ,i0,i02)
      call flow3d(v,d2v,wk(p2),wk(p3),wk(p4),wk(p5),wk(p6),ni,nj,nk,nit
     &     ,njt,dx,dy,dz,i0,i0t)
      return
      end

c*******************************************************************
c     setutop:  subtracts smoothed 2-D surface velocity from input utop
c        don't forget utop and u2d are complex!
c*******************************************************************

      subroutine setutop(utop,utop0,u2d,ni,nj,ni2d,i0,i02)
      implicit none
      integer ni,nj,ni2d
      real utop(2,ni,nj),utop0(ni,nj),u2d(2,ni2d)
      integer i0,i02

      integer i,j,is

      is=i02-i0

      do j=1,nj
         do i=1,ni
            utop(1,i,j)=utop0(i,j)-u2d(1,is+i)
            utop(2,i,j)=0.
         enddo
      enddo
      return
      end
      
c*******************************************************************
c     flow2d:  routine for Fourier Transform solution of 2-D problem
c     calculates and puts on 3-D grid
c*******************************************************************
      subroutine flow2d(v,d2v,utop,vt,d2vt,kx,ni,nj,nk,ni2t,dx,dz,i0,i02
     &     )
      implicit none
      integer ni,nj,nk,ni2t
      real v(3,ni,nj,nk),d2v(3,ni,nj,nk),utop(2,ni2t)
      real vt(2,ni2t,2), d2vt(2,ni2t,2)
      real kx(ni2t)
      integer i0,i02,di0
      real dx,dz

      real z,xm
      real pi
      integer i,j,k,nih

      data pi /3.14159265358979323846/
      di0=i02-i0
c---
c     calculate    kx
c---
      xm=ni2t*dx
      nih=ni2t/2+1
      do i=1,nih
         kx(i)=2.*pi*(i-1)/xm
      enddo
      do i=1,nih-2              ! reflect around nyquist freq
         kx(nih+i)=-kx(nih-i)
      enddo
c      call arrcopy(cpi,utop,2*ni2t)
      call four1(utop,ni2t,1)   ! FT of top velocity
c      call arrcopy(cpo,utop,2*ni2t)
c      call four1(cpo,ni2t,-1)
c      call arrmult(cpo,2*ni2t,1./ni2t)
c------
c     do that fourier thing
c---

      do k=1,nk                ! start filling in on planes
         z=dz*(nk-k)            !fill from bottom up and don't forget to 
                                ! flip W
         call setuw2d(vt,d2vt,utop,kx,ni2t,z)
         do j=1,nj
            do i=1,ni
               v(1,i,j,k)=vt(1,di0+i,1)
               v(3,i,j,k)=-vt(1,di0+i,2)
               d2v(1,i,j,k)=d2vt(1,di0+i,1)
               d2v(3,i,j,k)=-d2vt(1,di0+i,2)
            enddo
         enddo
      enddo
      return
      end

c*******************************************************************
c     setuw2d:  routine for Fourier Transform solution of 2-D problem
c     calculates v and d2vt(z) for single z by inverse FT
c*******************************************************************
      subroutine setuw2d(vt,d2vt,utop,kx,ni,z)
      implicit none
      integer ni
      real vt(2,ni,2),d2vt(2,ni,2),utop(2,ni)
      real kx(ni),z

      integer i
      real kz,ekz,kn,ini

c--
c     dc offset to zero ?
c--
      vt(1,1,1)=0.
      vt(2,1,1)=0.
      vt(1,1,2)=0.
      vt(2,1,2)=0.      
      d2vt(1,1,1)=0.
      d2vt(2,1,1)=0.
      d2vt(1,1,2)=0.
      d2vt(2,1,2)=0.

      do i=2,ni
         kz=kx(i)*z
         kn=abs(kx(i))
         if (kz.le.100.) then
            ekz=exp(-kn*z)
         else
            ekz=0.
         endif 
         vt(1,i,1)=utop(1,i)*(1.-kz)*ekz
         vt(2,i,1)=utop(2,i)*(1.-kz)*ekz
         vt(1,i,2)=-utop(2,i)*kz*ekz
         vt(2,i,2)=utop(1,i)*kz*ekz
         d2vt(1,i,1)=2.*utop(1,i)*kn*kn*ekz
         d2vt(2,i,1)=2.*utop(2,i)*kn*kn*ekz
         d2vt(1,i,2)=2.*utop(2,i)*kn*kx(i)*ekz
         d2vt(2,i,2)=-2.*utop(1,i)*kn*kx(i)*ekz
      enddo
c---
c     FT back and pray
c---
      do i=1,2
         call four1(vt(1,1,i),ni,-1) ! IFFT of velocity
         call four1(d2vt(1,1,i),ni,-1) ! IFFT of delsqV
      enddo
      ini=real(1./ni)
      call arrmult(vt,4*ni,ini)
      call arrmult(d2vt,4*ni,ini)
      return
      end
      
c*******************************************************************
c     flow3d:  routine for Fourier Transform solution of 3-D problem
c     calculates perturbation and adds to 3-D grid
c*******************************************************************
      subroutine flow3d(v,d2v,utop,vt,d2vt,kx,ky,ni,nj,nk,nit,njt,dx,dy
     &     ,dz,i0,i02)
      implicit none
      integer ni,nj,nk,nit,njt
      real v(3,ni,nj,nk),d2v(3,ni,nj,nk),utop(2,nit,njt)
c      real cpi(2,64,64),cpo(2,64,64)
      real vt(2,nit,njt,3), d2vt(2,nit,njt,3)
      real kx(nit),ky(njt)
      integer i0,i02,di0
      real dx,dy,dz

      real z,xm,ym
      real pi
      integer i,j,k,nih,njh
      integer nn(2)

      data pi /3.14159265358979323846/
      di0=i02-i0
      nn(1)=nit
      nn(2)=njt
c---
c     calculate    kx,ky
c---
      xm=nit*dx
      nih=nit/2+1
      do i=1,nih
         kx(i)=2.*pi*(i-1)/xm
      enddo
      do i=1,nih-2              ! reflect around nyquist freq
         kx(nih+i)=-kx(nih-i)
      enddo
      ym=njt*dy
      njh=njt/2+1
      do j=1,njh
         ky(j)=2.*pi*(j-1)/ym
      enddo
      do j=1,njh-2              ! reflect around nyquist freq
         ky(njh+j)=-ky(njh-j)
      enddo
c      call arrcopy(cpi,utop,2*nit*njt)
c---
c     2-D transform utop
c---      
      call fourn(utop,nn,2,1)   ! FT of top velocity
      utop(1,1,1)=0.  ! kill any residual DC component
      utop(2,1,1)=0.
c      call arrcopy(cpo,utop,2*nit*njt)
c      call fourn(cpo,nn,2,-1)
c      call arrmult(cpo,2*nit*njt,1./(nit*njt))
c------
c     do that fourier thing
c---

      do k=1,nk                ! start filling in on planes
         z=dz*(nk-k)            !fill from bottom up and don't forget to 
                                       ! flip W
         call setuvw3d(vt,d2vt,utop,kx,ky,nit,njt,z)
c         call arrcopy(cpi,vt,2*nit*njt)
         print *, 'k=',k
         do j=1,nj
            do i=1,ni
               v(1,i,j,k)=v(1,i,j,k)+vt(1,di0+i,j,1)
               v(2,i,j,k)=v(2,i,j,k)+vt(1,di0+i,j,2)
               v(3,i,j,k)=v(3,i,j,k)-vt(1,di0+i,j,3)
               d2v(1,i,j,k)=d2v(1,i,j,k)+d2vt(1,di0+i,j,1)
               d2v(2,i,j,k)=d2v(2,i,j,k)+d2vt(1,di0+i,j,2)
               d2v(3,i,j,k)=d2v(3,i,j,k)-d2vt(1,di0+i,j,3)
            enddo
         enddo
      enddo
      return
      end

c*******************************************************************
c     setuvw3d:  routine for Fourier Transform solution of 2-D problem
c     calculates v and d2vt(z) for single z by inverse FT
c*******************************************************************
      subroutine setuvw3d(vt,d2vt,utop,kx,ky,ni,nj,z)
      implicit none
      integer ni,nj
      real vt(2,ni,nj,3),d2vt(2,ni,nj,3),utop(2,ni,nj)
      real kx(ni),ky(nj),z

      integer i,j
      real kz,knz,ekz,kn,ini
      integer nn(2)

      nn(1)=ni
      nn(2)=nj
c--
c     dc offset to zero ?
c--
c      do k=1,3
c         do j=1,ni
c            do i=1,2
c               vt(i,j,1,k)=0.
c               d2vt(i,j,1,k)=0.
c            enddo
c         enddo
c      enddo
c      do k=1,3
c         do j=1,nj
c            do i=1,2
c               vt(i,1,j,k)=0.
c               d2vt(i,1,j,k)=0.
c            enddo
c         enddo
c      enddo      


      do j=1,nj
         do i=1,ni
            kz=kx(i)*z
            kn=sqrt(kx(i)*kx(i)+ky(j)*ky(j))
            if (kn.eq.0.) kn=1.
            knz=kn*z
            if (knz.le.100.) then
               ekz=exp(-knz)
            else
               ekz=0.
            endif 
            vt(1,i,j,1)=utop(1,i,j)*(1.-kx(i)*kz/kn)*ekz
            vt(2,i,j,1)=utop(2,i,j)*(1.-kx(i)*kz/kn)*ekz
            vt(1,i,j,2)=-utop(1,i,j)*ky(j)*kz/kn*ekz
            vt(2,i,j,2)=-utop(2,i,j)*ky(j)*kz/kn*ekz
            vt(1,i,j,3)=-utop(2,i,j)*kz*ekz
            vt(2,i,j,3)=utop(1,i,j)*kz*ekz
            d2vt(1,i,j,1)=2.*utop(1,i,j)*kx(i)*kx(i)*ekz
            d2vt(2,i,j,1)=2.*utop(2,i,j)*kx(i)*kx(i)*ekz
            d2vt(1,i,j,2)=2.*utop(1,i,j)*kx(i)*ky(j)*ekz
            d2vt(2,i,j,2)=2.*utop(2,i,j)*kx(i)*ky(j)*ekz
            d2vt(1,i,j,3)=2.*utop(2,i,j)*kn*kx(i)*ekz
            d2vt(2,i,j,3)=-2.*utop(1,i,j)*kn*kx(i)*ekz
         enddo
      enddo 
c---
c     FT back and pray
c---
      do i=1,3
         call fourn(vt(1,1,1,i),nn,2,-1) ! IFFT of velocity
         call fourn(d2vt(1,1,1,i),nn,2,-1) ! IFFT of delsqV
      enddo
      ini=real(1./(ni*nj))
      call arrmult(vt,6*ni*nj,ini)
      call arrmult(d2vt,6*ni*nj,ini)
      return
      end
      

 
